 /*
 *  file      : main.c
 *  project   : zoomc
 *  with      : Gtk-2
 *
 *  copyright : (C) 2010 - 2011 by Claude Bulin
 *
 *  ZOOMC - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of ZOOMC.
 * 
 *  ZOOMC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  ZOOMC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with ZOOMC.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../libzoomc/lib.h"
#endif

#include "global.h"
#include "win_about.h"
#include "poche.h"
#include "scan.h"
#include "configuser.h"



GtkBuilder	*GtkBuilderProjet = NULL;
GtkWidget	*WindMain = NULL;


KEYS keys = {
	FALSE,		// gboolean	BoolGDKPress
	FALSE,		// gboolean	BoolGDK_Control_L_R
	FALSE,		// gboolean	BoolGDK_Control_A
	0,		// guint	keyval
};



// ---------------------------------------------
// StructClass->Id = windmain
// ---------------------------------------------
void remove_memory_apply( void )
{
	g_print ("\n--------------------------------------------------\n");
	PRINT_FUNC_LF();
	g_print ("--------------------------------------------------\n");
	
	g_print ("Recuperation position et taille de la fenetre\n");
	gdk_window_get_root_origin (WindMain->window, &Config.WinPos_X, &Config.WinPos_Y);
	gdk_window_get_size (WindMain->window, &Config.WinWidth, &Config.WinHeight);	

	g_print ("Sauvegarde de la configuration\n");
	config_save ();
	
	g_print ("poche_remove_view()\n");
	poche_remove_view();
	
	g_print ("poche_remove_ListImage()\n");
	poche_remove_ListImage();
	
	g_print ("scan_remove_glist_media ()\n");
	scan_remove_glist_media ();

	g_print ("libcombo_remove ()\n");
	libcombo_remove ();

	g_print ("config_remove ()\n");
	config_remove ();

	Config.PathPochette = libutils_remove_temporary_rep( Config.PathPochette );
	
	if( NULL != Config.PathLoadImg ) {
		g_free( Config.PathLoadImg );
		Config.PathLoadImg = NULL;
	}
	if( Config.PathSaveImg ) {
		g_free( Config.PathSaveImg );
		Config.PathSaveImg = NULL;
	}
	
	g_print ("\n");
}
gboolean on_windmain_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	remove_memory_apply();
	gtk_main_quit ();
	return( FALSE );
}
gboolean on_windmain_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
	remove_memory_apply();
	gtk_main_quit ();
	return( FALSE );
}
void on_imagemenuitem_quitter_activate (GtkImageMenuItem *menuitem, gpointer user_data)
{
	remove_memory_apply();
	gtk_main_quit ();
}
void on_imagemenuitem_about_activate (GtkImageMenuItem *menuitem, gpointer user_data)
{
	WinAbout_open();
}

// 
// 
gboolean on_windmain_key_press_event( GtkWidget *widget, GdkEventKey *kevent, gpointer user_data )
{
	keys.BoolGDKPress = TRUE;
	keys.keyval       = kevent->keyval;
	
	if( kevent->keyval == GDK_Control_L || kevent->keyval == GDK_Control_R ) {
		keys.BoolGDK_Control_L_R = TRUE;
		pochedir_set_ctrl( TRUE );
	}		
	else if( kevent->state == 20 || kevent->state == 21 ) {
		if( kevent->keyval == 65 || kevent->keyval == 97 ) {
			keys.BoolGDK_Control_A = TRUE;
		}
	}
	if( kevent->keyval == GDK_Delete) {
		// poche_remove_image();
	}

	return FALSE;
}
// 
// 
gboolean on_windmain_key_release_event( GtkWidget *widget, GdkEventKey *kevent, gpointer user_data )
{
	keys.BoolGDKPress        = FALSE;
	keys.BoolGDK_Control_L_R = FALSE;
	keys.BoolGDK_Control_A   = FALSE;
	keys.keyval              = 0;
	
	pochedir_set_ctrl( FALSE );
	
	return FALSE;
}

/* Install un 'handler' sur une mauvaise allocation memoire
*  --
*  entree : int sig :
*  retour : -
*/
void segfault_handler(int sig)
{
	g_print ("\n=========================================================\n");
	g_print ("=  Segmentation fault\n");
	g_print (_("=  Adressage memoire invalide\n"));
	g_print ("=-------------------------------------------------------=\n");
	g_print ("=      You've probably found a bug in ZOOMC\n");
	g_print (_("=      Vous avez probablement trouve un bug dans ZOOMC\n"));
	g_print ("=========================================================\n\n");
		
	exit (EXIT_FAILURE);
}

/* gint main (gint argc, gchar *argv[], gchar **envp)
 *
 * argc  designe le nombre d'arguments transmis au moment du lancement de l'executable
 * argv  designe le vecteur contenant les differents arguments
 * envp  designe le vecteur contenant les informations sur l'environnement
 */
gint main (gint argc, gchar *argv[], gchar **envp)
{
	GError	*p_err = NULL;
	gchar	*CurrentDir = g_get_current_dir ();
	gchar	*PathCurrentDir = NULL;
	gchar	*PathShareProjet = NULL;
	gchar	*Path = NULL;
	gchar	*PackageVersion = NULL;

	g_print ("%s :: %s (line = %d)\n", __FILE__, __FUNCTION__, __LINE__);
	g_print ("\n");
	g_print ("***************************\n");
	g_print ("%s %s\n", PACKAGE, VERSION);
	g_print ("***************************\n");
	g_print ("%s %s compiled at %s %s\n", PACKAGE, VERSION, __DATE__, __TIME__);
	g_print ("Compiler: gcc %s\n", __VERSION__);
	g_print ("***************************\n");
	g_print ("Gtk version  = %d.%d.%d\n", GTK_MAJOR_VERSION, GTK_MICRO_VERSION, GTK_MINOR_VERSION);
	g_print ("Glib version = %d.%d.%d\n", GLIB_MAJOR_VERSION, GLIB_MICRO_VERSION, GLIB_MINOR_VERSION);
	g_print ("***************************\n\n");
#ifdef ENABLE_NLS
	g_print ("ENABLE_NLS OK\n");
	g_print ("\tGETTEXT_PACKAGE    = %s\n", GETTEXT_PACKAGE);
	g_print ("\tLOCALE_DIR         = %s\n", LOCALE_DIR);

	/* IF Segmentation fault */
	signal (SIGSEGV, segfault_handler);

	config_read();

	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#else
	// g_error ("ENABLE_NLS IS NOT OK\n\n");
#endif
	gtk_set_locale ();
	gtk_init (&argc, &argv);

	if (NULL == (GtkBuilderProjet = gtk_builder_new ())) {
		return (EXIT_FAILURE);
	}
	gtk_builder_set_translation_domain (GtkBuilderProjet, NULL);

	PathCurrentDir = g_strdup_printf ("%s/glade/%s", CurrentDir, PACKAGE_NAME_GLADE);
	g_free (CurrentDir);
	CurrentDir = NULL;
	PathShareProjet = g_strdup_printf ("%s/%s/glade/%s", DATA_DIR, PACKAGE_NAME, PACKAGE_NAME_GLADE);

	if (FALSE == g_file_test (PathCurrentDir, G_FILE_TEST_EXISTS) && FALSE == g_file_test (PathShareProjet, G_FILE_TEST_EXISTS)) {
		g_print("!---------------------------------------------------------------------------------------------------\n");
		g_print("! LE FICHIER XML: [ %s ] N'EXISTE PAS !!!\n", PathCurrentDir);
		g_print("!---------------------------------------------------------------------------------------------------\n");
		g_print("! LE FICHIER XML: [ %s ] N'EXISTE PAS !!!\n", PathShareProjet);
		g_print("!---------------------------------------------------------------------------------------------------\n");

		g_free (PathCurrentDir);
		PathCurrentDir = NULL;
		g_free (PathShareProjet);
		PathShareProjet = NULL;

		return (EXIT_FAILURE);
	}
	else if (TRUE == g_file_test (PathCurrentDir, G_FILE_TEST_EXISTS)) {

		Path = PathCurrentDir;
	}
	else if (TRUE == g_file_test (PathShareProjet, G_FILE_TEST_EXISTS)) {

		Path = PathShareProjet;
	}
	g_print("\nPATHNAME FILE %s: %s\n\n", PACKAGE_NAME_GLADE, Path);
	if (gtk_builder_add_from_file (GtkBuilderProjet, Path, &p_err) <= 0) {
		g_warning ("Couldn't load builder file: %s", p_err->message);
		g_error_free (p_err);

		g_free (PathCurrentDir);
		PathCurrentDir = NULL;
		g_free (PathShareProjet);
		PathShareProjet = NULL;
		Path = NULL;

		return (EXIT_FAILURE);
	}
	g_free (PathCurrentDir);
	PathCurrentDir = NULL;
	g_free (PathShareProjet);
	PathShareProjet = NULL;
	Path = NULL;

	gtk_builder_set_translation_domain  (GtkBuilderProjet, gtk_set_locale ());
	gtk_builder_connect_signals (GtkBuilderProjet, NULL);
	WindMain = GTK_WIDGET (gtk_builder_get_object (GtkBuilderProjet, "windmain"));

	// REDIMENSIONNER LA FENETRE
	gtk_window_resize(GTK_WINDOW(WindMain), Config.WinWidth, Config.WinHeight);
	// PLACER LA FENETRE
	gtk_window_move(GTK_WINDOW(WindMain), Config.WinPos_X, Config.WinPos_Y);

	PackageVersion = g_strdup_printf ("%s %s", PACKAGE, VERSION);
	gtk_window_set_title (GTK_WINDOW (WindMain), PackageVersion);
	g_free (PackageVersion);
	PackageVersion = NULL;

	gtk_widget_show (WindMain);
	gtk_main ();

	exit(EXIT_SUCCESS);
}

